@@ -33,48 +33,48 @@ class ErrorMiddleware(object):
 
     Settings:
 
-    ``debug``:
-        If true, then tracebacks will be shown in the browser.
+      ``debug``:
+          If true, then tracebacks will be shown in the browser.
 
-    ``error_email``:
-        an email address (or list of addresses) to send exception 
-        reports to
+      ``error_email``:
+          an email address (or list of addresses) to send exception 
+          reports to
 
-    ``error_log``:
-        a filename to append tracebacks to
+      ``error_log``:
+          a filename to append tracebacks to
 
-    ``show_exceptions_in_wsgi_errors``:
-        If true, then errors will be printed to ``wsgi.errors`` 
-        (frequently a server error log, or stderr).
+      ``show_exceptions_in_wsgi_errors``:
+          If true, then errors will be printed to ``wsgi.errors`` 
+          (frequently a server error log, or stderr).
 
-    ``from_address``, ``smtp_server``, ``error_subject_prefix``:
-        variables to control the emailed exception reports
+      ``from_address``, ``smtp_server``, ``error_subject_prefix``:
+          variables to control the emailed exception reports
 
-    ``error_message``:
-        When debug mode is off, the error message to show to users.
+      ``error_message``:
+          When debug mode is off, the error message to show to users.
 
-    ``xmlhttp_key``:
-        When this key (default ``_``) is in the request GET variables
-        (not POST!), expect that this is an XMLHttpRequest, and the
-        response should be more minimal; it should not be a complete
-        HTML page.
+      ``xmlhttp_key``:
+          When this key (default ``_``) is in the request GET variables
+          (not POST!), expect that this is an XMLHttpRequest, and the
+          response should be more minimal; it should not be a complete
+          HTML page.
 
     Environment Configuration:
     
-    ``paste.throw_errors``:
-        If this setting in the request environment is true, then this
-        middleware is disabled. This can be useful in a testing situation
-        where you don't want errors to be caught and transformed.
+      ``paste.throw_errors``:
+          If this setting in the request environment is true, then this
+          middleware is disabled. This can be useful in a testing situation
+          where you don't want errors to be caught and transformed.
 
-    ``paste.expected_exceptions``:
-        When this middleware encounters an exception listed in this
-        environment variable and when the ``start_response`` has not 
-        yet occurred, the exception will be re-raised instead of being
-        caught.  This should generally be set by middleware that may 
-        (but probably shouldn't be) installed above this middleware, 
-        and wants to get certain exceptions.  Exceptions raised after
-        ``start_response`` have been called are always caught since
-        by definition they are no longer expected.
+      ``paste.expected_exceptions``:
+          When this middleware encounters an exception listed in this
+          environment variable and when the ``start_response`` has not 
+          yet occurred, the exception will be re-raised instead of being
+          caught.  This should generally be set by middleware that may 
+          (but probably shouldn't be) installed above this middleware, 
+          and wants to get certain exceptions.  Exceptions raised after
+          ``start_response`` have been called are always caught since
+          by definition they are no longer expected.
 
     """
 
@@ -95,7 +95,7 @@ class ErrorMiddleware(object):
             global_conf = {}
         if debug is NoDefault:
             debug = global_conf.get('debug')
-        from paste.deploy import converters
+        from paste.util import converters
         self.debug_mode = converters.asbool(debug)
         if error_email is None:
             error_email = (global_conf.get('error_email')
@@ -138,7 +138,7 @@ class ErrorMiddleware(object):
         try:
             __traceback_supplement__ = Supplement, self, environ
             app_iter = self.application(environ, detect_start_response)
-            return self.catching_iter(app_iter, environ)
+            return self.make_catching_iter(app_iter, environ)
         except:
             exc_info = sys.exc_info()
             try:
@@ -158,29 +158,11 @@ class ErrorMiddleware(object):
                 # clean up locals...
                 exc_info = None
 
-    def catching_iter(self, app_iter, environ):
-        __traceback_supplement__ = Supplement, self, environ
-        if not app_iter:
-            raise StopIteration
-        error_on_close = False
-        try:
-            for v in app_iter:
-                yield v
-            if hasattr(app_iter, 'close'):
-                error_on_close = True
-                app_iter.close()
-        except:
-            response = self.exception_handler(sys.exc_info(), environ)
-            if not error_on_close and hasattr(app_iter, 'close'):
-                try:
-                    app_iter.close()
-                except:
-                    close_response = self.exception_handler(
-                        sys.exc_info(), environ)
-                    response += (
-                        '<hr noshade>Error in .close():<br>%s'
-                        % close_response)
-            yield response
+    def make_catching_iter(self, app_iter, environ):
+        if isinstance(app_iter, (list, tuple)):
+            # These don't raise
+            return app_iter
+        return CatchingIter(app_iter, environ, self)
 
     def exception_handler(self, exc_info, environ):
         simple_html_error = False
@@ -201,6 +183,68 @@ class ErrorMiddleware(object):
             error_message=self.error_message,
             simple_html_error=simple_html_error)
 
+class CatchingIter(object):
+
+    """
+    A wrapper around the application iterator that will catch
+    exceptions raised by the a generator, or by the close method, and
+    display or report as necessary.
+    """
+
+    def __init__(self, app_iter, environ, error_middleware):
+        self.app_iterable = app_iter
+        self.app_iterator = iter(app_iter)
+        self.environ = environ
+        self.error_middleware = error_middleware
+        self.closed = False
+
+    def __iter__(self):
+        return self
+
+    def next(self):
+        __traceback_supplement__ = (
+            Supplement, self.error_middleware, self.environ)
+        if self.closed:
+            raise StopIteration
+        try:
+            return self.app_iterator.next()
+        except StopIteration:
+            self.closed = True
+            close_response = self._close()
+            if close_response is not None:
+                return close_response
+            else:
+                raise StopIteration
+        except:
+            self.closed = True
+            close_response = self._close()
+            response = self.error_middleware.exception_handler(
+                sys.exc_info(), self.environ)
+            if close_response is not None:
+                response += (
+                    '<hr noshade>Error in .close():<br>%s'
+                    % close_response)
+            return response
+
+    def close(self):
+        # This should at least print something to stderr if the
+        # close method fails at this point
+        if not self.closed:
+            self._close()
+
+    def _close(self):
+        """Close and return any error message"""
+        if not hasattr(self.app_iterable, 'close'):
+            return None
+        try:
+            self.app_iterable.close()
+            return None
+        except:
+            close_response = self.error_middleware.exception_handler(
+                sys.exc_info(), self.environ)
+            return close_response
+
+
 class Supplement(object):
 
     """
@@ -334,7 +378,7 @@ def handle_exception(exc_info, error_stream, html=True,
             An error occurred.  See the error logs for more information.
             (Turn debug on to display exception reports here)
             '''
-            return_error = error_template('', error_message, '')
+            return_error = error_template('', msg, '')
     else:
         return_error = None
     if not reported and error_stream:
@@ -378,3 +422,13 @@ def error_template(head_html, exception, extra):
     %s
     </body>
     </html>''' % (head_html, exception, extra)
+
+def make_error_middleware(app, global_conf, **kw):
+    return ErrorMiddleware(app, global_conf=global_conf, **kw)
+
+doc_lines = ErrorMiddleware.__doc__.splitlines(True)
+for i in range(len(doc_lines)):
+    if doc_lines[i].strip().startswith('Settings'):
+        make_error_middleware.__doc__ = ''.join(doc_lines[i:])
+        break
+del i, doc_lines
