@@ -11,7 +11,8 @@ Gzip-encodes the response.
 """
 
 import gzip
-from paste.response import header_value
+from paste.response import header_value, remove_header
+from paste.httpheaders import CONTENT_LENGTH
 
 try:
     from cStringIO import StringIO
@@ -47,8 +48,10 @@ class GzipResponse(object):
         self.compress_level = compress_level
         self.buffer = StringIO()
         self.compressible = False
+        self.content_length = None
 
     def gzip_start_response(self, status, headers, exc_info=None):
+        self.headers = headers
         ct = header_value(headers,'content-type')
         ce = header_value(headers,'content-encoding')
         self.compressible = False
@@ -59,7 +62,10 @@ class GzipResponse(object):
             self.compressible = False
         if self.compressible:
             headers.append(('content-encoding', 'gzip'))
-        return self.start_response(status, headers, exc_info)
+        remove_header(headers, 'content-length')
+        self.headers = headers
+        self.status = status
+        return self.buffer.write
 
     def write(self):
         out = self.buffer
@@ -82,6 +88,9 @@ class GzipResponse(object):
         finally:
             if hasattr(app_iter, 'close'):
                 app_iter.close()
+        content_length = self.buffer.tell()
+        CONTENT_LENGTH.update(self.headers, content_length)
+        self.start_response(self.status, self.headers)
 
 def filter_factory(application, **conf):
     import warnings
