@@ -17,7 +17,9 @@ if pyOpenSSL is installed, it also provides SSL capabilities.
 # @@: add support for chunked encoding, this is not a 1.1 server
 #     till this is completed.
 
-import socket, sys, threading, urlparse, Queue
+import atexit
+import socket, sys, threading, urlparse, Queue, urllib
+import posixpath
 from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
 from SocketServer import ThreadingMixIn
 from paste.util import converters
@@ -160,6 +162,8 @@ class WSGIHandlerMixin:
         """
 
         (_, _, path, query, fragment) = urlparse.urlsplit(self.path)
+        path = urllib.unquote(path)
+        path = posixpath.normpath(path)
         (server_name, server_port) = self.server.server_address
 
         rfile = self.rfile
@@ -181,7 +185,7 @@ class WSGIHandlerMixin:
                ,'PATH_INFO': path
                ,'QUERY_STRING': query
                ,'CONTENT_TYPE': self.headers.get('Content-Type', '')
-               ,'CONTENT_LENGTH': self.headers.get('Content-Length', '')
+               ,'CONTENT_LENGTH': self.headers.get('Content-Length', '0')
                ,'SERVER_NAME': server_name
                ,'SERVER_PORT': str(server_port)
                ,'SERVER_PROTOCOL': self.request_version
@@ -311,6 +315,29 @@ else:
                 conn = _ConnFixer(conn)
             return (conn, info)
 
+    def _auto_ssl_context():
+        import OpenSSL, time, random
+        pkey = OpenSSL.crypto.PKey()
+        pkey.generate_key(OpenSSL.crypto.TYPE_RSA, 768)
+
+        cert = OpenSSL.crypto.X509()
+
+        cert.set_serial_number(random.randint(0, sys.maxint))
+        cert.gmtime_adj_notBefore(0)
+        cert.gmtime_adj_notAfter(60 * 60 * 24 * 365)
+        cert.get_subject().CN = '*'
+        cert.get_subject().O = 'Dummy Certificate'
+        cert.get_issuer().CN = 'Untrusted Authority'
+        cert.get_issuer().O = 'Self-Signed'
+        cert.set_pubkey(pkey)
+        cert.sign(pkey, 'md5')
+
+        ctx = SSL.Context(SSL.SSLv23_METHOD)
+        ctx.use_privatekey(pkey)
+        ctx.use_certificate(cert)
+
+        return ctx
+
 class WSGIHandler(WSGIHandlerMixin, BaseHTTPRequestHandler):
     """
     A WSGI handler that overrides POST, GET and HEAD to delegate
@@ -333,7 +360,7 @@ class ThreadPool(object):
     """
     SHUTDOWN = object()
 
-    def __init__(self, nworkers, name="ThreadPool"):
+    def __init__(self, nworkers, name="ThreadPool", daemon=False):
         """
         Create thread pool with `nworkers` worker threads.
         """
@@ -344,9 +371,13 @@ class ThreadPool(object):
         for i in range(self.nworkers):
             worker = threading.Thread(target=self.worker_thread_callback,
                                       name=("%s worker %d" % (self.name, i)))
+            worker.setDaemon(daemon)
             worker.start()
             self.workers.append(worker)
 
+        if not daemon:
+            atexit.register(self.shutdown)
+
     def worker_thread_callback(self):
         """
         Worker thread should call this method to get and process queued
@@ -374,13 +405,13 @@ class ThreadPoolMixIn:
     """
     Mix-in class to process requests from a thread pool
     """
-    def __init__(self, nworkers):
+    def __init__(self, nworkers, daemon=False):
         # Create and start the workers
         self.running = True
         assert nworkers > 0, "ThreadPoolMixin servers must have at least one worker"
         self.thread_pool = ThreadPool(nworkers,
             "ThreadPoolMixin HTTP server on %s:%d"
-                % (self.server_name, self.server_port))
+                % (self.server_name, self.server_port), daemon)
 
     def process_request(self, request, client_address):
         """
@@ -437,6 +468,7 @@ class ThreadPoolMixIn:
         """
         self.running = False
         self.socket.close()
+        self.thread_pool.shutdown()
 
 class WSGIServerBase(SecureHTTPServer):
     def __init__(self, wsgi_application, server_address,
@@ -458,15 +490,16 @@ class WSGIServer(ThreadingMixIn, WSGIServerBase):
 
 class WSGIThreadPoolServer(ThreadPoolMixIn, WSGIServerBase):
     def __init__(self, wsgi_application, server_address,
-                 RequestHandlerClass=None, ssl_context=None, nworkers=10):
+                 RequestHandlerClass=None, ssl_context=None,
+                 nworkers=10, daemon_threads=False):
         WSGIServerBase.__init__(self, wsgi_application, server_address,
                                 RequestHandlerClass, ssl_context)
-        ThreadPoolMixIn.__init__(self, nworkers)
+        ThreadPoolMixIn.__init__(self, nworkers, daemon_threads)
 
 def serve(application, host=None, port=None, handler=None, ssl_pem=None,
-          server_version=None, protocol_version=None, start_loop=True,
-          daemon_threads=None, socket_timeout=None, use_threadpool=True,
-          threadpool_workers=10):
+          ssl_context=None, server_version=None, protocol_version=None,
+          start_loop=True, daemon_threads=None, socket_timeout=None,
+          use_threadpool=True, threadpool_workers=10):
     """
     Serves your ``application`` over HTTP(S) via WSGI interface
 
@@ -488,8 +521,10 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
 
     ``ssl_pem``
 
-        This an optional SSL certificate file (via OpenSSL) You can
-        generate a self-signed test PEM certificate file as follows:
+        This an optional SSL certificate file (via OpenSSL). You can
+        supply ``*`` and a development-only certificate will be
+        created for you, or you can generate a self-signed test PEM
+        certificate file as follows:
 
             $ openssl genrsa 1024 > host.key
             $ chmod 400 host.key
@@ -498,6 +533,13 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
             $ cat host.cert host.key > host.pem
             $ chmod 400 host.pem
 
+    ``ssl_context``
+
+        This an optional SSL context object for the server.  A SSL
+        context will be automatically constructed for you if you supply
+        ``ssl_pem``.  Supply this to use a context of your own
+        construction.
+
     ``server_version``
 
         The version of the server as reported in HTTP response line. This
@@ -547,13 +589,18 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
         Number of worker threads to create when ``use_threadpool`` is true. This
         can be a string or an integer value.
     """
-    ssl_context = None
-    if ssl_pem:
+    is_ssl = False
+    if ssl_pem or ssl_context:
         assert SSL, "pyOpenSSL is not installed"
+        is_ssl = True
         port = int(port or 4443)
-        ssl_context = SSL.Context(SSL.SSLv23_METHOD)
-        ssl_context.use_privatekey_file(ssl_pem)
-        ssl_context.use_certificate_file(ssl_pem)
+        if not ssl_context:
+            if ssl_pem == '*':
+                ssl_context = _auto_ssl_context()
+            else:
+                ssl_context = SSL.Context(SSL.SSLv23_METHOD)
+                ssl_context.use_privatekey_file(ssl_pem)
+                ssl_context.use_certificate_file(ssl_pem)
 
     host = host or '127.0.0.1'
     if not port:
@@ -575,7 +622,8 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
 
     if converters.asbool(use_threadpool):
         server = WSGIThreadPoolServer(application, server_address, handler,
-                                      ssl_context, int(threadpool_workers))
+                                      ssl_context, int(threadpool_workers),
+                                      daemon_threads)
     else:
         server = WSGIServer(application, server_address, handler, ssl_context)
         if daemon_threads:
@@ -585,7 +633,13 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
         server.wsgi_socket_timeout = int(socket_timeout)
 
     if converters.asbool(start_loop):
-        print "serving on %s:%s" % server.server_address
+        protocol = is_ssl and 'https' or 'http'
+        host, port = server.server_address
+        if host == '0.0.0.0':
+            print 'serving on 0.0.0.0:%s view at %s://127.0.0.1:%s' % \
+                (port, protocol, port)
+        else:
+            print "serving on %s://%s:%s" % (protocol, host, port)
         try:
             server.serve_forever()
         except KeyboardInterrupt:
