@@ -31,6 +31,7 @@ TODO:
 
 import httplib
 import urlparse
+import urllib
 
 from paste import httpexceptions
 from paste.util.converters import aslist
@@ -87,21 +88,24 @@ class Proxy(object):
         else:
             body = ''
             
+        path_info = urllib.quote(environ['PATH_INFO'])
         if self.path:            
-            request_path = environ['PATH_INFO']
+            request_path = path_info
             if request_path[0] == '/':
                 request_path = request_path[1:]
                 
             path = urlparse.urljoin(self.path, request_path)
         else:
-            path = environ['PATH_INFO']
+            path = path_info
             
         conn.request(environ['REQUEST_METHOD'],
                      path,
                      body, headers)
         res = conn.getresponse()
         headers_out = []
-        for header, value in res.getheaders():
+        for full_header in res.msg.headers:
+            header, value = full_header.split(':', 1)
+            value = value.strip()
             if header.lower() not in filtered_headers:
                 headers_out.append((header, value))
                 
@@ -161,6 +165,12 @@ class TransparentProxy(object):
         self.force_host = force_host
         self.force_scheme = force_scheme
 
+    def __repr__(self):
+        return '<%s %s force_host=%r force_scheme=%r>' % (
+            self.__class__.__name__,
+            hex(id(self)),
+            self.force_host, self.force_scheme)
+
     def __call__(self, environ, start_response):
         scheme = environ['wsgi.url_scheme']
         if self.force_host is None:
@@ -205,13 +215,16 @@ class TransparentProxy(object):
         
         path = (environ.get('SCRIPT_NAME', '')
                 + environ.get('PATH_INFO', ''))
+        path = urllib.quote(path)
         if 'QUERY_STRING' in environ:
             path += '?' + environ['QUERY_STRING']
         conn.request(environ['REQUEST_METHOD'],
                      path, body, headers)
         res = conn.getresponse()
         headers_out = []
-        for header, value in res.getheaders():
+        for full_header in res.msg.headers:
+            header, value = full_header.split(':', 1)
+            value = value.strip()
             if header.lower() not in filtered_headers:
                 headers_out.append((header, value))
                 
