@@ -7,7 +7,6 @@ WSGI applications that parse the URL and dispatch to on-disk resources
 import os
 import sys
 import imp
-import urllib
 import pkg_resources
 import mimetypes
 from paste import request
@@ -203,8 +202,7 @@ class URLParser(object):
                 # None of the index files found
                 filename = None
         else:
-            # Handle quoted chars (e.g. %20)
-            filename = self.find_file(environ, urllib.unquote(name))
+            filename = self.find_file(environ, name)
         if filename is None:
             return None, filename
         else:
@@ -436,9 +434,12 @@ class StaticURLParser(object):
         if root_directory is not None:
             self.root_directory = os.path.normpath(self.root_directory)
         else:
-            self.root_directory = os.path.normpath(directory)
+            self.root_directory = directory
         self.cache_max_age = cache_max_age
-        
+        if os.path.sep != '/':
+            directory = directory.replace('/', os.path.sep)
+            self.root_directory = self.root_directory.replace('/', os.path.sep)
+
     def __call__(self, environ, start_response):
         path_info = environ.get('PATH_INFO', '')
         if not path_info:
@@ -447,9 +448,10 @@ class StaticURLParser(object):
             # @@: This should obviously be configurable
             filename = 'index.html'
         else:
-            # Handle quoted chars (e.g. %20)
-            filename = urllib.unquote(request.path_info_pop(environ))
+            filename = request.path_info_pop(environ)
         full = os.path.normpath(os.path.join(self.directory, filename))
+        if os.path.sep != '/':
+            full = full.replace('/', os.path.sep)
         if self.root_directory is not None and not full.startswith(self.root_directory):
             # Out of bounds
             return self.not_found(environ, start_response)
@@ -504,7 +506,7 @@ class StaticURLParser(object):
         return exc.wsgi_application(environ, start_response)
 
     def error_extra_path(self, environ, start_response):
-        exc = httpexceptions.HTTPBadRequest(
+        exc = httpexceptions.HTTPNotFound(
             'The trailing path %r is not allowed' % environ['PATH_INFO'])
         return exc.wsgi_application(environ, start_response)
     
@@ -552,8 +554,7 @@ class PkgResourcesParser(StaticURLParser):
             # @@: This should obviously be configurable
             filename = 'index.html'
         else:
-            # Handle quoted chars (e.g. %20)
-            filename = urllib.unquote(request.path_info_pop(environ))
+            filename = request.path_info_pop(environ)
         resource = os.path.normpath(self.resource_name + '/' + filename)
         if self.root_resource is not None and not resource.startswith(self.root_resource):
             # Out of bounds
