@@ -5,8 +5,6 @@ sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
 
 import pkg_resources
 pkg_resources.require('Paste')
-pkg_resources.require('PasteDeploy')
-pkg_resources.require('PasteScript')
 
 import py
 
